// Desktop USB Test.cpp : Defines the entry point for the console application.
//
// This application opens the USB ActiveSync Comm Driver for communication with a WinCE Device.
// It sends 40 DWORDs and then reads 40 DWORDs, from values (0 - 39).
//
// ADS# 72002-12121
// jnosek 9-19-2006

#include "stdafx.h"
#include <Windows.h>
#include <stdio.h>

int _tmain(int argc, _TCHAR* argv[])
{
	HANDLE hUsbPort = NULL;

	// USB Com Driver Name
	WCHAR portName[] = TEXT("\\\\.\\wceusbsh001");
	
	DWORD bytesWritten;
	DWORD bytesRead;
	DWORD buffer;
	DWORD eventBuffer;
	DWORD errorBuffer;

	// COM State Configuration Structure
	DCB state;

	// Comstats Structure, used to hold data from ClearCommError function
	COMSTAT comstats;

	int i = 0;
	int returnVal;
	char keyPress;

	wprintf(TEXT("Desktop USB Test   ADS# 720020-12121\r\n"));

	wprintf(TEXT("Opening USB Port %s ...\r\n"), portName);

	// Open the USB Com Port
	hUsbPort = CreateFile(	portName, 
							GENERIC_WRITE | GENERIC_READ, 
							0, 
							NULL, 
							OPEN_EXISTING, 
							FILE_ATTRIBUTE_NORMAL, 
							NULL);

	if (hUsbPort == INVALID_HANDLE_VALUE) {
		wprintf(TEXT("Could not open port %s\r\n"), portName);
		printf("GetLastError returned %d\r\n", GetLastError());

		Sleep(2000);
		return -1;
	}
	else
	{
		wprintf(TEXT("... Opened USB Port %s\r\n"), portName);
	}

	// Configure the DCB stucture for the COM driver
	// Settings of importance:
	// No flow control,
	// Max baudrate,
	// 8 bit data, no parity, 1 stop bit
	state.DCBlength = sizeof(DCB);
	state.BaudRate = CBR_256000;
	state.fBinary = TRUE;
	state.fOutxCtsFlow = FALSE;
	state.fOutxDsrFlow = FALSE;
	state.fDtrControl = DTR_CONTROL_DISABLE;
	state.fRtsControl = RTS_CONTROL_DISABLE;
	state.fDsrSensitivity = FALSE;
	state.fTXContinueOnXoff = FALSE;
	state.fOutX = FALSE;
	state.fInX = FALSE;
	state.fErrorChar = FALSE;
	state.fNull = FALSE;
	state.fRtsControl = FALSE;
	state.fAbortOnError = FALSE;
	state.wReserved = 0;
	state.ByteSize = 8;
	state.Parity = NOPARITY;
	state.StopBits = ONESTOPBIT;
	state.XonLim = 1;
	state.XoffLim = 1;

	// Set Comm State
	SetCommState(hUsbPort, &state);

	// Set DTR high to trigger the RS232_NOTFICATION_EVENT on the WinCE Device
	EscapeCommFunction(hUsbPort, SETDTR);
	
	Sleep(1000);

	// Write 40 DWORDs
	for(buffer = 0; buffer < 40; buffer++)
	{
		printf("%d)", buffer);
		returnVal = WriteFile(hUsbPort, &buffer, sizeof(buffer), &bytesWritten, NULL);
				
		// Check returnVal
		if(returnVal != 0)
		{
			printf("Data Written (%d): 0x%x\r\n", bytesWritten, buffer);
		}
		else
		{
			wprintf(TEXT("Write Error on port %s\r\n"), portName);
			printf("GetLastError returned %d\r\n", GetLastError());
		}
	}

	printf("\r\n");
	buffer = 0;

	// Set Comm Mask to trigger upon recieving characters
	SetCommMask(hUsbPort, EV_RXCHAR);

	// Read 40 DWORDs (looking for the sent value of 39)
	while(buffer < 39)
	{
		// Wait for CommEvent on Com Port
		WaitCommEvent(hUsbPort, &eventBuffer, NULL);

		returnVal = ReadFile(hUsbPort, &buffer, sizeof(buffer), &bytesRead, NULL);

		// Check returnVal
		if(returnVal != 0)
		{
			wprintf(TEXT("%d) Data Read (%d): 0x%x\r\n"), i, bytesRead, buffer);
			i++;
		}
		else
		{
			wprintf(TEXT("Read Error on port %s\r\n"), portName);
			wprintf(TEXT("GetLastError returned %d\r\n"), GetLastError());
		}

		// Check for additional data in queue
		ClearCommError(hUsbPort, &errorBuffer, &comstats);

		// While bytes still left in the queue
		while(comstats.cbInQue != 0)
		{
			returnVal = ReadFile(hUsbPort, &buffer, sizeof(buffer), &bytesRead, NULL);
			wprintf(TEXT("++%d) Data Read (%d): 0x%x\r\n"), i, bytesRead, buffer);
			i++;

			// Check returnVal
			if(returnVal != 0)
			{
				wprintf(TEXT("%d) Data Read (%d): 0x%x\r\n"), i, bytesRead, buffer);
				i++;
			}
			else
			{
				wprintf(TEXT("Read Error on port %s\r\n"), portName);
				wprintf(TEXT("GetLastError returned %d\r\n"), GetLastError());
			}

			// Check for additonal data in queue
			ClearCommError(hUsbPort, &errorBuffer, &comstats);
		}
	}

	// Clear DTR for end of data transmission
	EscapeCommFunction(hUsbPort, CLRDTR);

	printf("Press ENTER to quit...\r\n");
	scanf("%c", &keyPress);
	
	return 0;
}